package com.zsc.edu.gateway.framework;

import com.zsc.edu.gateway.modules.system.entity.Dept;
import com.zsc.edu.gateway.modules.system.entity.User;
import com.zsc.edu.gateway.modules.system.vo.DeptTree;
import com.zsc.edu.gateway.modules.system.vo.UserTree;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Component
public class DeptTreeUtil {

    public static <E> List<E> makeTree(List<E> list, Predicate<E> rootCheck, BiFunction<E, E, Boolean> parentCheck, BiConsumer<E, List<E>> setSubChildren) {
        return list.stream()
                .filter(rootCheck)
                .peek(x -> setSubChildren.accept(x, makeChildren(x, list, parentCheck, setSubChildren)))
                .collect(Collectors.toList());
    }

    private static <E> List<E> makeChildren(E parent, List<E> allData, BiFunction<E, E, Boolean> parentCheck, BiConsumer<E, List<E>> setSubChildren) {
        return allData.stream()
                .filter(x -> parentCheck.apply(parent, x))
                .peek(x -> setSubChildren.accept(x, makeChildren(x, allData, parentCheck, setSubChildren)))
                .collect(Collectors.toList());
    }

    public static List<DeptTree> buildDeptTree(List<Dept> depots, Map<Long, List<User>> userMap) {
        List<DeptTree> deptTrees = depots.stream()
                .map(DeptTreeUtil::convertToDeptTree)
                .collect(Collectors.toList());

        deptTrees.forEach(deptTree -> {
            List<User> users = userMap.getOrDefault(deptTree.getId(), Collections.emptyList());
            deptTree.setMembers(users.stream()
                    .map(DeptTreeUtil::convertToUserTree)
                    .collect(Collectors.toList()));
        });

        return makeTree(
                deptTrees,
                deptTree -> deptTree.getPid() == null || deptTree.getPid() == 0L,
                (parent, child) -> parent.getId().equals(child.getPid()),
                DeptTree::setChildren
        );
    }

    private static DeptTree convertToDeptTree(Dept dept) {
        DeptTree deptTree = new DeptTree();
        deptTree.setId(dept.getId());
        deptTree.setPid(dept.getPid());
        deptTree.setName(dept.getName());
        return deptTree;
    }

    private static UserTree convertToUserTree(User user) {
        UserTree userTree = new UserTree();
        userTree.setId(user.getId());
        userTree.setName(user.getName());
        return userTree;
    }
}
